(function ($, Drupal, drupalSettings) {
    /**
     * Disable the continue buttons in the checkout process once they are clicked
     * and provide a notification to the user.
     */
    Drupal.behaviors.commerceStripeWebPaymentRequest = {
        attach: function (context) {
            /**
             * Detect what browser we're on.
             */

            var browser = (function (agent) {
                switch (true) {
                    case agent.indexOf("edge") > -1:
                        return "ie";
                    case agent.indexOf("msie") > -1:
                        return "ie";
                    case agent.indexOf("opr") > -1 && !!window.opr:
                        return "opera";
                    case agent.indexOf("chrome") > -1 && !!window.chrome:
                        return "chrome";
                    case agent.indexOf("trident") > -1:
                        return "ie";
                    case agent.indexOf("firefox") > -1:
                        return "firefox";
                    case agent.indexOf("safari") > -1:
                        return "safari";
                    default:
                        return "other";
                }
            })(window.navigator.userAgent.toLowerCase());

            var $form = $(drupalSettings.stripe_payment_request.form_id);
            // Check if Stripe Payment Request tab active.

            // Create our Payment Request button.
            var stripe = Stripe(drupalSettings.stripe_payment_request.publishable_key);
            var paymentRequest = stripe.paymentRequest(drupalSettings.stripe_payment_request.paymentRequest);

            $form.once('stripe-payment-request-setup-wrapper').each(function () {
                var elements = stripe.elements();
                var prButton = elements.create('paymentRequestButton', {
                    paymentRequest: paymentRequest,
                    style: {
                        paymentRequestButton: {
                            height: '44px',
                            theme: 'light-outline',
                            type: 'buy',
                        },
                    },
                });


                // Check if we can make payments.
                paymentRequest.canMakePayment().then(function (result) {

                    // If we can make Stripe Apple/Google payments, show the button.
                    // Only NULL means you can't use it, return with applePay true/false is only meant for apple pay
                    if (result) {
                        prButton.mount('#payment-request-button');
                        //Drupal.behaviors.commerceStripeWebPaymentRequest.showButton();
                    }
                    // Else, if they can't make payments, show the setup info.
                    else {
                        document.getElementById('payment-request-button').style.display = 'none';
                        //Drupal.behaviors.commerceStripeWebPaymentRequest.showSetUpButton();
                    }
                });

                // Listen for a token, process and complete the payment on our end.
                paymentRequest.on('token', function (ev) {
                    // Send the token to the server to charge it!
                    // Prepare data for backend call.
                    var charge_data = {
                        token: ev.token.id,
                        result: ev,
                        order_id: drupalSettings.stripe_payment_request.order_id,
                        payment_request: drupalSettings.stripe_payment_request.paymentRequest,
                        serialized_form: $(drupalSettings.stripe_payment_request.form_id).formSerialize()
                    };

                    var charge_path = drupalSettings.path.baseUrl + drupalSettings.path.pathPrefix + drupalSettings.stripe_payment_request.charge_path;

                    // Send the card details to the server so we can charge the card and
                    // complete the payment.
                    fetch(charge_path, {
                        method: 'POST',
                        body: JSON.stringify({data: charge_data}),
                        headers: {'content-type': 'application/json'},
                    })
                        .then(function (response) {
                            if (response.ok) {
                                ev.complete('success');
                                // Examine the text in the response.
                                response.json().then(function (data) {
                                    var alter_event = jQuery.Event('stripePaymentRequestPaySuccess');
                                    alter_event.charge_data = charge_data;
                                    alter_event.server_response = data;
                                    $('#payment-request-button').trigger(alter_event);
                                    // Redirect to complete checkout page.
                                    window.location.href = data.redirect_uri;
                                });
                            } else {
                                ev.complete('fail');
                                // Examine the text in the response.
                                response.json().then(function (data) {
                                    var alter_event = jQuery.Event('stripePaymentRequestPayError');
                                    alter_event.error_message = data;
                                    $('#payment-request-button').trigger(alter_event);
                                });
                            }
                        });
                });

                paymentRequest.on('shippingaddresschange', function(ev) {
                        // Perform server-side request to fetch shipping options
                        var shipping_path = drupalSettings.path.baseUrl + drupalSettings.path.pathPrefix + drupalSettings.stripe_payment_request.shipping_path;
                        fetch(shipping_path, {
                            method: 'POST',
                            body: JSON.stringify({shippingAddress: ev.shippingAddress, order_id: drupalSettings.stripe_payment_request.order_id}),
                            headers: {'content-type': 'application/json'},
                        }).then(function(response) {
                            return response.json();
                        }).then(function(result) {
                            ev.updateWith({
                                status: 'success',
                                shippingOptions: result.shippingOptions,
                            });
                        });
                });
            });
        },
    };
})(jQuery, Drupal, drupalSettings);
